# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/rsm/rsm.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List, Optional

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class SliceType(betterproto.Enum):
    SLICE_TYPE_DL_SLICE = 0
    SLICE_TYPE_UL_SLICE = 1


class SchedulerType(betterproto.Enum):
    SCHEDULER_TYPE_ROUND_ROBIN = 0
    SCHEDULER_TYPE_PROPORTIONALLY_FAIR = 1
    SCHEDULER_TYPE_QOS_BASED = 2


class UeIdType(betterproto.Enum):
    UE_ID_TYPE_CU_UE_F1_AP_ID = 0
    UE_ID_TYPE_DU_UE_F1_AP_ID = 1
    UE_ID_TYPE_RAN_UE_NGAP_ID = 2
    UE_ID_TYPE_AMF_UE_NGAP_ID = 3
    UE_ID_TYPE_ENB_UE_S1_AP_ID = 4


@dataclass(eq=False, repr=False)
class SliceItem(betterproto.Message):
    e2_node_id: str = betterproto.string_field(1)
    slice_ids: List[str] = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class Ack(betterproto.Message):
    success: bool = betterproto.bool_field(1)
    cause: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CreateSliceRequest(betterproto.Message):
    e2_node_id: str = betterproto.string_field(1)
    slice_id: str = betterproto.string_field(2)
    scheduler_type: "SchedulerType" = betterproto.enum_field(3)
    weight: str = betterproto.string_field(4)
    slice_type: "SliceType" = betterproto.enum_field(5)


@dataclass(eq=False, repr=False)
class CreateSliceResponse(betterproto.Message):
    ack: "Ack" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateSliceRequest(betterproto.Message):
    e2_node_id: str = betterproto.string_field(1)
    slice_id: str = betterproto.string_field(2)
    scheduler_type: "SchedulerType" = betterproto.enum_field(3)
    weight: str = betterproto.string_field(4)
    slice_type: "SliceType" = betterproto.enum_field(5)


@dataclass(eq=False, repr=False)
class UpdateSliceResponse(betterproto.Message):
    ack: "Ack" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DeleteSliceRequest(betterproto.Message):
    e2_node_id: str = betterproto.string_field(1)
    slice_id: str = betterproto.string_field(2)
    slice_type: "SliceType" = betterproto.enum_field(3)


@dataclass(eq=False, repr=False)
class DeleteSliceResponse(betterproto.Message):
    ack: "Ack" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SliceAssocItem(betterproto.Message):
    ue_slice_assoc_id: str = betterproto.string_field(1)
    e2_node_id: str = betterproto.string_field(2)
    ue_id: List["UeIdType"] = betterproto.enum_field(3)
    slice_id: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class UeId(betterproto.Message):
    ue_id: str = betterproto.string_field(1)
    type: "UeIdType" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class SetUeSliceAssociationRequest(betterproto.Message):
    e2_node_id: str = betterproto.string_field(1)
    ue_id: List["UeId"] = betterproto.message_field(2)
    dl_slice_id: str = betterproto.string_field(3)
    ul_slice_id: str = betterproto.string_field(4)
    drb_id: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class SetUeSliceAssociationResponse(betterproto.Message):
    ack: "Ack" = betterproto.message_field(1)
    assigned_ue_slice_assoc_id: str = betterproto.string_field(2)


class RsmStub(betterproto.ServiceStub):
    async def create_slice(
        self,
        *,
        e2_node_id: str = "",
        slice_id: str = "",
        scheduler_type: "SchedulerType" = None,
        weight: str = "",
        slice_type: "SliceType" = None,
    ) -> "CreateSliceResponse":

        request = CreateSliceRequest()
        request.e2_node_id = e2_node_id
        request.slice_id = slice_id
        request.scheduler_type = scheduler_type
        request.weight = weight
        request.slice_type = slice_type

        return await self._unary_unary(
            "/onos.rsm.Rsm/CreateSlice", request, CreateSliceResponse
        )

    async def update_slice(
        self,
        *,
        e2_node_id: str = "",
        slice_id: str = "",
        scheduler_type: "SchedulerType" = None,
        weight: str = "",
        slice_type: "SliceType" = None,
    ) -> "UpdateSliceResponse":

        request = UpdateSliceRequest()
        request.e2_node_id = e2_node_id
        request.slice_id = slice_id
        request.scheduler_type = scheduler_type
        request.weight = weight
        request.slice_type = slice_type

        return await self._unary_unary(
            "/onos.rsm.Rsm/UpdateSlice", request, UpdateSliceResponse
        )

    async def delete_slice(
        self,
        *,
        e2_node_id: str = "",
        slice_id: str = "",
        slice_type: "SliceType" = None,
    ) -> "DeleteSliceResponse":

        request = DeleteSliceRequest()
        request.e2_node_id = e2_node_id
        request.slice_id = slice_id
        request.slice_type = slice_type

        return await self._unary_unary(
            "/onos.rsm.Rsm/DeleteSlice", request, DeleteSliceResponse
        )

    async def set_ue_slice_association(
        self,
        *,
        e2_node_id: str = "",
        ue_id: Optional[List["UeId"]] = None,
        dl_slice_id: str = "",
        ul_slice_id: str = "",
        drb_id: str = "",
    ) -> "SetUeSliceAssociationResponse":
        ue_id = ue_id or []

        request = SetUeSliceAssociationRequest()
        request.e2_node_id = e2_node_id
        if ue_id is not None:
            request.ue_id = ue_id
        request.dl_slice_id = dl_slice_id
        request.ul_slice_id = ul_slice_id
        request.drb_id = drb_id

        return await self._unary_unary(
            "/onos.rsm.Rsm/SetUeSliceAssociation",
            request,
            SetUeSliceAssociationResponse,
        )


class RsmBase(ServiceBase):
    async def create_slice(
        self,
        e2_node_id: str,
        slice_id: str,
        scheduler_type: "SchedulerType",
        weight: str,
        slice_type: "SliceType",
    ) -> "CreateSliceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update_slice(
        self,
        e2_node_id: str,
        slice_id: str,
        scheduler_type: "SchedulerType",
        weight: str,
        slice_type: "SliceType",
    ) -> "UpdateSliceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_slice(
        self, e2_node_id: str, slice_id: str, slice_type: "SliceType"
    ) -> "DeleteSliceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_ue_slice_association(
        self,
        e2_node_id: str,
        ue_id: Optional[List["UeId"]],
        dl_slice_id: str,
        ul_slice_id: str,
        drb_id: str,
    ) -> "SetUeSliceAssociationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create_slice(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "e2_node_id": request.e2_node_id,
            "slice_id": request.slice_id,
            "scheduler_type": request.scheduler_type,
            "weight": request.weight,
            "slice_type": request.slice_type,
        }

        response = await self.create_slice(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_update_slice(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "e2_node_id": request.e2_node_id,
            "slice_id": request.slice_id,
            "scheduler_type": request.scheduler_type,
            "weight": request.weight,
            "slice_type": request.slice_type,
        }

        response = await self.update_slice(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_slice(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "e2_node_id": request.e2_node_id,
            "slice_id": request.slice_id,
            "slice_type": request.slice_type,
        }

        response = await self.delete_slice(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_set_ue_slice_association(
        self, stream: grpclib.server.Stream
    ) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "e2_node_id": request.e2_node_id,
            "ue_id": request.ue_id,
            "dl_slice_id": request.dl_slice_id,
            "ul_slice_id": request.ul_slice_id,
            "drb_id": request.drb_id,
        }

        response = await self.set_ue_slice_association(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.rsm.Rsm/CreateSlice": grpclib.const.Handler(
                self.__rpc_create_slice,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateSliceRequest,
                CreateSliceResponse,
            ),
            "/onos.rsm.Rsm/UpdateSlice": grpclib.const.Handler(
                self.__rpc_update_slice,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpdateSliceRequest,
                UpdateSliceResponse,
            ),
            "/onos.rsm.Rsm/DeleteSlice": grpclib.const.Handler(
                self.__rpc_delete_slice,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteSliceRequest,
                DeleteSliceResponse,
            ),
            "/onos.rsm.Rsm/SetUeSliceAssociation": grpclib.const.Handler(
                self.__rpc_set_ue_slice_association,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetUeSliceAssociationRequest,
                SetUeSliceAssociationResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/snapshot/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import timedelta

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase


class Phase(betterproto.Enum):
    """Phase is a snapshot phase"""

    # MARK is the first phase in which changes are marked for deletion
    MARK = 0
    # DELETE is the second phase in which changes are deleted from stores
    DELETE = 1


class State(betterproto.Enum):
    """State is the state of a snapshot within a phase"""

    # PENDING indicates the snapshot is pending
    PENDING = 0
    # RUNNING indicates the snapshot is in progress
    RUNNING = 1
    # COMPLETE indicates the snapshot is complete
    COMPLETE = 2


@dataclass(eq=False, repr=False)
class Status(betterproto.Message):
    """Status is the status of a snapshot"""

    # 'phase' is the snapshot phase
    phase: "Phase" = betterproto.enum_field(1)
    # 'state' is the state of a snapshot
    state: "State" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class RetentionOptions(betterproto.Message):
    """RetentionOptions specifies the retention policy for a change log"""

    # 'retain_window' is the duration for which to retain network changes
    retain_window: timedelta = betterproto.message_field(1)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/v2/transaction/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase


class State(betterproto.Enum):
    """State is the state of a phase"""

    # PENDING indicates the phase is pending
    PENDING = 0
    # COMPLETE indicates the phase is complete
    COMPLETE = 2
    # FAILED indicates the phase failed
    FAILED = 3


class Phase(betterproto.Enum):
    """Phase is the phase of a Transaction"""

    # CHANGE indicates the transaction has been requested
    CHANGE = 0
    # ROLLBACK indicates a rollback has been requested for the transaction
    ROLLBACK = 1


class Reason(betterproto.Enum):
    """Reason is a reason for a FAILED state"""

    # NONE indicates no error has occurred
    NONE = 0
    # ERROR indicates an error occurred when applying the change
    ERROR = 1


@dataclass(eq=False, repr=False)
class Transaction(betterproto.Message):
    """Transaction refers to a multi-target transactional change"""

    # 'id' is the unique identifier of the transaction This field should be set
    # prior to persisting the object.
    id: str = betterproto.string_field(1)
    # 'index' is a monotonically increasing, globally unique index of the change
    # The index is provided by the store, is static and unique for each unique
    # change identifier, and should not be modified by client code.
    index: int = betterproto.uint64_field(2)
    # 'revision' is the change revision number The revision number is provided by
    # the store and should not be modified by client code. Each unique state of
    # the change will be assigned a unique revision number which can be used for
    # optimistic concurrency control when updating or deleting the change state.
    revision: int = betterproto.uint64_field(3)
    # 'status' is the current lifecycle status of the transaction
    status: "Status" = betterproto.message_field(4)
    # 'created' is the time at which the transaction was created
    created: datetime = betterproto.message_field(5)
    # 'updated' is the time at which the transaction was last updated
    updated: datetime = betterproto.message_field(6)
    # 'changes' is a set of changes to apply to targets The list of changes
    # should contain only a single change per target/version pair.
    changes: List["Change"] = betterproto.message_field(7)
    # 'deleted' is a flag indicating whether this transaction is being deleted by
    # a snapshot
    deleted: bool = betterproto.bool_field(9)
    # 'dependency' is a reference to the transaction on which this transaction is
    # dependent
    dependency: "TransactionRef" = betterproto.message_field(10)
    # 'dependents' is a list of references to transactions that depend on this
    # transaction
    dependents: List["TransactionRef"] = betterproto.message_field(11)
    # 'username' is the name of the user that made the transaction
    username: str = betterproto.string_field(12)


@dataclass(eq=False, repr=False)
class TransactionRef(betterproto.Message):
    """TransactionRef is a reference to a transaction"""

    none: "betterproto_lib_google_protobuf.Empty" = betterproto.message_field(
        1, group="id"
    )
    transaction_id: str = betterproto.string_field(2, group="id")


@dataclass(eq=False, repr=False)
class Change(betterproto.Message):
    """Change represents a configuration change to a single target"""

    # 'target_id' is the identifier of the target to which this change applies
    target_id: str = betterproto.string_field(1)
    # 'target_version' is an optional target version to which to apply this
    # change
    target_version: str = betterproto.string_field(2)
    # 'target_type' is an optional target type to which to apply this change
    target_type: str = betterproto.string_field(3)
    # 'values' is a set of change values to apply
    values: List["ChangeValue"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class ChangeValue(betterproto.Message):
    """
    ChangeValue is an individual Path/Value and removed flag combination in a
    Change
    """

    # 'path' is the path to change
    path: str = betterproto.string_field(1)
    # 'value' is the change value
    value: "__v2__.TypedValue" = betterproto.message_field(2)
    # 'removed' indicates whether this is a delete
    removed: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class Status(betterproto.Message):
    """Status is the status of a Transaction"""

    # 'phase' is the current phase of the
    phase: "Phase" = betterproto.enum_field(1)
    # 'state' is the state of the transaction within a Phase
    state: "State" = betterproto.enum_field(2)
    # 'reason' is a failure reason
    reason: "Reason" = betterproto.enum_field(3)
    # message is a result message
    message: str = betterproto.string_field(4)
    # MastershipState mastership info
    mastership_state: "MastershipState" = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class MastershipState(betterproto.Message):
    """Mastership state"""

    term: int = betterproto.uint64_field(1)
    node_id: str = betterproto.string_field(2)


from ... import v2 as __v2__
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/ransim/trafficsim/trafficsim.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class Type(betterproto.Enum):
    """Change event type"""

    # NONE indicates this response does not represent a modification of the
    # Change
    NONE = 0
    # ADDED is an event which occurs when a Change is added to the topology
    ADDED = 1
    # UPDATED is an event which occurs when a Change is updated
    UPDATED = 2
    # REMOVED is an event which occurs when a Change is removed from the
    # configuration
    REMOVED = 3


class UpdateType(betterproto.Enum):
    """
    In the case of an update this helps qualify the update type - add items as
    necessary
    """

    NOUPDATETYPE = 0
    # POSITION An update in position of UE only, without a change in the Tower
    # affinity
    POSITION = 1
    # TOWER An update in non-serving tower affinity (and also position) of UE
    TOWER = 2
    # HANDOVER An update of the serving tower
    HANDOVER = 3


@dataclass(eq=False, repr=False)
class MapLayoutRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListRoutesRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class ListRoutesResponse(betterproto.Message):
    # route is the route change on which the event occurred
    route: "_types__.Route" = betterproto.message_field(1)
    # type is a qualification of the type of change being made
    type: "Type" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ListUesRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListUesResponse(betterproto.Message):
    # Ue is the UserEquipment change on which the event occurred
    ue: "_types__.Ue" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class WatchUesRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class WatchUesResponse(betterproto.Message):
    # Ue is the UserEquipment change on which the event occurred
    ue: "_types__.Ue" = betterproto.message_field(1)
    # type is a qualification of the type of change being made
    type: "Type" = betterproto.enum_field(2)
    # update_type is a qualification of the type of UE change
    update_type: "UpdateType" = betterproto.enum_field(3)


@dataclass(eq=False, repr=False)
class SetNumberUEsRequest(betterproto.Message):
    number: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class SetNumberUEsResponse(betterproto.Message):
    number: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class ResetMetricsMsg(betterproto.Message):
    pass


class TrafficStub(betterproto.ServiceStub):
    async def get_map_layout(self) -> "_types__.MapLayout":

        request = MapLayoutRequest()

        return await self._unary_unary(
            "/onos.ransim.trafficsim.Traffic/GetMapLayout", request, _types__.MapLayout
        )

    async def list_routes(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["ListRoutesResponse"]:

        request = ListRoutesRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.trafficsim.Traffic/ListRoutes",
            request,
            ListRoutesResponse,
        ):
            yield response

    async def list_ues(self) -> AsyncIterator["ListUesResponse"]:

        request = ListUesRequest()

        async for response in self._unary_stream(
            "/onos.ransim.trafficsim.Traffic/ListUes",
            request,
            ListUesResponse,
        ):
            yield response

    async def watch_ues(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["WatchUesResponse"]:

        request = WatchUesRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.trafficsim.Traffic/WatchUes",
            request,
            WatchUesResponse,
        ):
            yield response

    async def set_number_u_es(self, *, number: int = 0) -> "SetNumberUEsResponse":

        request = SetNumberUEsRequest()
        request.number = number

        return await self._unary_unary(
            "/onos.ransim.trafficsim.Traffic/SetNumberUEs",
            request,
            SetNumberUEsResponse,
        )

    async def reset_metrics(self) -> "ResetMetricsMsg":

        request = ResetMetricsMsg()

        return await self._unary_unary(
            "/onos.ransim.trafficsim.Traffic/ResetMetrics", request, ResetMetricsMsg
        )


class TrafficBase(ServiceBase):
    async def get_map_layout(self) -> "_types__.MapLayout":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_routes(
        self, no_replay: bool, no_subscribe: bool
    ) -> AsyncIterator["ListRoutesResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_ues(self) -> AsyncIterator["ListUesResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_ues(
        self, no_replay: bool, no_subscribe: bool
    ) -> AsyncIterator["WatchUesResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_number_u_es(self, number: int) -> "SetNumberUEsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def reset_metrics(self) -> "ResetMetricsMsg":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_map_layout(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_map_layout(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_routes(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "no_replay": request.no_replay,
            "no_subscribe": request.no_subscribe,
        }

        await self._call_rpc_handler_server_stream(
            self.list_routes,
            stream,
            request_kwargs,
        )

    async def __rpc_list_ues(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.list_ues,
            stream,
            request_kwargs,
        )

    async def __rpc_watch_ues(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "no_replay": request.no_replay,
            "no_subscribe": request.no_subscribe,
        }

        await self._call_rpc_handler_server_stream(
            self.watch_ues,
            stream,
            request_kwargs,
        )

    async def __rpc_set_number_u_es(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "number": request.number,
        }

        response = await self.set_number_u_es(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_reset_metrics(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.reset_metrics(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.ransim.trafficsim.Traffic/GetMapLayout": grpclib.const.Handler(
                self.__rpc_get_map_layout,
                grpclib.const.Cardinality.UNARY_UNARY,
                MapLayoutRequest,
                _types__.MapLayout,
            ),
            "/onos.ransim.trafficsim.Traffic/ListRoutes": grpclib.const.Handler(
                self.__rpc_list_routes,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListRoutesRequest,
                ListRoutesResponse,
            ),
            "/onos.ransim.trafficsim.Traffic/ListUes": grpclib.const.Handler(
                self.__rpc_list_ues,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListUesRequest,
                ListUesResponse,
            ),
            "/onos.ransim.trafficsim.Traffic/WatchUes": grpclib.const.Handler(
                self.__rpc_watch_ues,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchUesRequest,
                WatchUesResponse,
            ),
            "/onos.ransim.trafficsim.Traffic/SetNumberUEs": grpclib.const.Handler(
                self.__rpc_set_number_u_es,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetNumberUEsRequest,
                SetNumberUEsResponse,
            ),
            "/onos.ransim.trafficsim.Traffic/ResetMetrics": grpclib.const.Handler(
                self.__rpc_reset_metrics,
                grpclib.const.Cardinality.UNARY_UNARY,
                ResetMetricsMsg,
                ResetMetricsMsg,
            ),
        }


from .. import types as _types__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/topo/config.proto, onos/topo/ran.proto, onos/topo/topo.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from datetime import datetime
from typing import AsyncIterator, Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class Protocol(betterproto.Enum):
    """Protocol to interact with a device"""

    # UNKNOWN_PROTOCOL constant needed to go around proto3 nullifying the 0
    # values
    UNKNOWN_PROTOCOL = 0
    # GNMI protocol reference
    GNMI = 1
    # P4RUNTIME protocol reference
    P4RUNTIME = 2
    # GNOI protocol reference
    GNOI = 3
    # E2 Control Plane Protocol
    E2AP = 4


class ConnectivityState(betterproto.Enum):
    """
    ConnectivityState represents the L3 reachability of a device from the
    service container (e.g. enos-config), independently of gRPC or the service
    itself (e.g. gNMI)
    """

    # UNKNOWN_CONNECTIVITY_STATE constant needed to go around proto3 nullifying
    # the 0 values
    UNKNOWN_CONNECTIVITY_STATE = 0
    # REACHABLE indicates the the service can reach the device at L3
    REACHABLE = 1
    # UNREACHABLE indicates the the service can't reach the device at L3
    UNREACHABLE = 2


class ChannelState(betterproto.Enum):
    """
    ConnectivityState represents the state of a gRPC channel to the device from
    the service container
    """

    # UNKNOWN_CHANNEL_STATE constant needed to go around proto3 nullifying the 0
    # values
    UNKNOWN_CHANNEL_STATE = 0
    # CONNECTED indicates the corresponding grpc channel is connected on this
    # device
    CONNECTED = 1
    # DISCONNECTED indicates the corresponding grpc channel is not connected on
    # this device
    DISCONNECTED = 2


class ServiceState(betterproto.Enum):
    """
    ServiceState represents the state of the gRPC service (e.g. gNMI) to the
    device from the service container
    """

    # UNKNOWN_SERVICE_STATE constant needed to go around proto3 nullifying the 0
    # values
    UNKNOWN_SERVICE_STATE = 0
    # AVAILABLE indicates the corresponding grpc service is available
    AVAILABLE = 1
    # UNAVAILABLE indicates the corresponding grpc service is not available
    UNAVAILABLE = 2
    # CONNECTING indicates the corresponding protocol is in the connecting phase
    # on this device
    CONNECTING = 3


class RanEntityKinds(betterproto.Enum):
    """
    TODO: Deprecate! Use language specific constant definitions instead. Kinds
    of RAN entities
    """

    E2NODE = 0
    E2CELL = 1
    E2T = 3
    XAPP = 4
    A1T = 5


class RanRelationKinds(betterproto.Enum):
    """
    TODO: Deprecate! Use language specific constant definitions instead. Kinds
    of RAN relations
    """

    CONTROLS = 0
    CONTAINS = 1
    NEIGHBORS = 2


class CellGlobalIdType(betterproto.Enum):
    NRCGI = 0
    ECGI = 1


class NodeType(betterproto.Enum):
    """
    NodeType is for use in encoding node type in topo Entity URI identifier;
    see section 9.2.6 of ORAN E2AP v02.00 spec
    """

    NT_NONE = 0
    NT_GNB = 1
    NT_EN_GNB = 2
    NT_NG_ENB = 3
    NT_ENB = 4


class ComponentType(betterproto.Enum):
    """
    ComponentType is for use in encoding node component type in topo Entity URI
    identifier; see section 9.2.6 of ORAN E2AP v02.00 spec
    """

    CT_NONE = 0
    CT_CU = 1
    CT_CU_UP = 2
    CT_DU = 3
    CT_ENB = 4


class E2SmRsmCommand(betterproto.Enum):
    E2_SM_RSM_COMMAND_SLICE_CREATE = 0
    E2_SM_RSM_COMMAND_SLICE_UPDATE = 1
    E2_SM_RSM_COMMAND_SLICE_DELETE = 2
    E2_SM_RSM_COMMAND_UE_ASSOCIATE = 3
    E2_SM_RSM_COMMAND_EVENT_TRIGGERS = 4


class RsmSlicingType(betterproto.Enum):
    SLICING_TYPE_STATIC = 0
    SLICING_TYPE_DYNAMIC = 1


class RsmSchedulerType(betterproto.Enum):
    SCHEDULER_TYPE_ROUND_ROBIN = 0
    SCHEDULER_TYPE_PROPORTIONALLY_FAIR = 1
    SCHEDULER_TYPE_QOS_BASED = 2


class RsmSliceType(betterproto.Enum):
    SLICE_TYPE_DL_SLICE = 0
    SLICE_TYPE_UL_SLICE = 1


class UeIdType(betterproto.Enum):
    UE_ID_TYPE_CU_UE_F1_AP_ID = 0
    UE_ID_TYPE_DU_UE_F1_AP_ID = 1
    UE_ID_TYPE_RAN_UE_NGAP_ID = 2
    UE_ID_TYPE_AMF_UE_NGAP_ID = 3
    UE_ID_TYPE_ENB_UE_S1_AP_ID = 4


class InterfaceType(betterproto.Enum):
    INTERFACE_UNKNOWN = 0
    INTERFACE_E2T = 1
    INTERFACE_E2AP101 = 2
    INTERFACE_E2AP200 = 3


class EventType(betterproto.Enum):
    """EventType is a topo operation event type"""

    NONE = 0
    ADDED = 1
    UPDATED = 2
    REMOVED = 3


class RelationFilterScope(betterproto.Enum):
    TARGET_ONLY = 0
    ALL = 1
    SOURCE_AND_TARGET = 2


class SortOrder(betterproto.Enum):
    UNORDERED = 0
    ASCENDING = 1
    DESCENDING = 2


class ObjectType(betterproto.Enum):
    UNSPECIFIED = 0
    ENTITY = 1
    RELATION = 2
    KIND = 3


@dataclass(eq=False, repr=False)
class Asset(betterproto.Message):
    """Basic asset information"""

    name: str = betterproto.string_field(1)
    manufacturer: str = betterproto.string_field(2)
    model: str = betterproto.string_field(3)
    serial: str = betterproto.string_field(4)
    asset: str = betterproto.string_field(5)
    sw_version: str = betterproto.string_field(6)
    role: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class Configurable(betterproto.Message):
    """Configurable device aspect"""

    type: str = betterproto.string_field(1)
    address: str = betterproto.string_field(2)
    target: str = betterproto.string_field(3)
    version: str = betterproto.string_field(4)
    timeout: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class MastershipState(betterproto.Message):
    """Aspect for tracking device mastership"""

    term: int = betterproto.uint64_field(1)
    node_id: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class TlsOptions(betterproto.Message):
    """TLS connectivity aspect"""

    insecure: bool = betterproto.bool_field(1)
    plain: bool = betterproto.bool_field(2)
    key: str = betterproto.string_field(3)
    ca_cert: str = betterproto.string_field(4)
    cert: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class AdHoc(betterproto.Message):
    """Aspect for ad-hoc properties"""

    properties: Dict[str, str] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass(eq=False, repr=False)
class ProtocolState(betterproto.Message):
    """
    ProtocolState contains information related to service and connectivity to a
    device
    """

    # The protocol to which state relates
    protocol: "Protocol" = betterproto.enum_field(1)
    # ConnectivityState contains the L3 connectivity information
    connectivity_state: "ConnectivityState" = betterproto.enum_field(2)
    # ChannelState relates to the availability of the gRPC channel
    channel_state: "ChannelState" = betterproto.enum_field(3)
    # ServiceState indicates the availability of the gRPC servic on top of the
    # channel
    service_state: "ServiceState" = betterproto.enum_field(4)


@dataclass(eq=False, repr=False)
class Protocols(betterproto.Message):
    """Protocols"""

    state: List["ProtocolState"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Location(betterproto.Message):
    # Legacy Geographical location; expected value type of "location" aspect
    lat: float = betterproto.double_field(1)
    lng: float = betterproto.double_field(2)
    wgs84: "Wgs84Location" = betterproto.message_field(3, group="ext")
    cartesian: "CartesianLocation" = betterproto.message_field(4, group="ext")

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.lat:
            warnings.warn("Location.lat is deprecated", DeprecationWarning)
        if self.lng:
            warnings.warn("Location.lng is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class Wgs84Location(betterproto.Message):
    # Latitude in degrees, between +/- 90.
    latitude_deg: float = betterproto.double_field(1)
    # Longitude in degrees, between +/- 180.
    longitude_deg: float = betterproto.double_field(2)
    # Altitude in meters, from the surface of the WGS-84 ellipsoid.
    altitude_m: float = betterproto.double_field(3)


@dataclass(eq=False, repr=False)
class CartesianLocation(betterproto.Message):
    """ECEF-compatible cartesian coordinates."""

    # X, Y, and Z coordinates in meters.
    x_m: float = betterproto.double_field(1)
    y_m: float = betterproto.double_field(2)
    z_m: float = betterproto.double_field(3)


@dataclass(eq=False, repr=False)
class AzElOrientation(betterproto.Message):
    # Azimuth angle in degrees.
    azimuth_deg: float = betterproto.double_field(1)
    # Elevation angle in degrees.
    elevation_deg: float = betterproto.double_field(2)
    # Rotation around boresight in degrees.
    rotation_deg: float = betterproto.double_field(3)


@dataclass(eq=False, repr=False)
class YprOrientation(betterproto.Message):
    # Yaw angle around z-axis.
    yaw_deg: float = betterproto.double_field(1)
    # Pitch angle around y-axix.
    pitch_deg: float = betterproto.double_field(2)
    # Roll angle around x-axis.
    roll_deg: float = betterproto.double_field(3)


@dataclass(eq=False, repr=False)
class Orientation(betterproto.Message):
    azel: "AzElOrientation" = betterproto.message_field(1, group="orientation")
    ypr: "YprOrientation" = betterproto.message_field(2, group="orientation")


@dataclass(eq=False, repr=False)
class Waypoint(betterproto.Message):
    time: datetime = betterproto.message_field(1)
    location: "Location" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Waypoints(betterproto.Message):
    """A set of waypoints that can be interpolated."""

    waypoint: List["Waypoint"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class OrbitData(betterproto.Message):
    """
    Minimal satellite orbit data, e.g. from a TLE, that can be propagated.
    """

    epoch: datetime = betterproto.message_field(1)
    # Orbit inclination in degrees.
    inclination_deg: float = betterproto.double_field(2)
    # Right ascension of the ascending node in degrees.
    raan_deg: float = betterproto.double_field(3)
    # Orbit eccentricity.
    e: float = betterproto.double_field(4)
    # Argument of perigee in degrees.
    argument_deg: float = betterproto.double_field(5)
    # Mean anomaly in degrees.
    anomaly_deg: float = betterproto.double_field(6)
    # Mean motion in revolutions per day.
    mean_motion: float = betterproto.double_field(7)


@dataclass(eq=False, repr=False)
class Motion(betterproto.Message):
    fixed_location: "Location" = betterproto.message_field(1, group="motion")
    waypoints: "Waypoints" = betterproto.message_field(2, group="motion")
    orbit: "OrbitData" = betterproto.message_field(3, group="motion")


@dataclass(eq=False, repr=False)
class Coverage(betterproto.Message):
    """Area of coverage; expected value type of "coverage" aspect"""

    height: int = betterproto.int32_field(1)
    arc_width: int = betterproto.int32_field(2)
    azimuth: int = betterproto.int32_field(3)
    tilt: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class E2Node(betterproto.Message):
    """
    E2Node aspect; expected value type of "E2NODE" aspect and expected on
    entities of "E2NODE" kind
    """

    service_models: Dict[str, "ServiceModelInfo"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass(eq=False, repr=False)
class E2NodeConfig(betterproto.Message):
    """E2NodeConfig aspect tracks the managed configuration for an E2 node"""

    connections: List["Interface"] = betterproto.message_field(1)
    version: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class Lease(betterproto.Message):
    """Lease aspect with an expiration timestamp for RAN entities"""

    expiration: datetime = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Interface(betterproto.Message):
    """Interface determines address and type of an endpoint interface"""

    type: "InterfaceType" = betterproto.enum_field(1)
    ip: str = betterproto.string_field(2)
    port: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class E2TInfo(betterproto.Message):
    """E2TInfo E2T aspect;"""

    interfaces: List["Interface"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CellGlobalId(betterproto.Message):
    value: str = betterproto.string_field(1)
    type: "CellGlobalIdType" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class NeighborCellId(betterproto.Message):
    cell_global_id: "CellGlobalId" = betterproto.message_field(1)
    plmn_id: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class E2Cell(betterproto.Message):
    """
    E2Cell aspect; expected value type of "E2CELL" aspect and expected on
    entities of "E2CELL" kind
    """

    cell_object_id: str = betterproto.string_field(1)
    cell_global_id: "CellGlobalId" = betterproto.message_field(2)
    antenna_count: int = betterproto.uint32_field(3)
    earfcn: int = betterproto.uint32_field(4)
    cell_type: str = betterproto.string_field(5)
    pci: int = betterproto.uint32_field(6)
    kpi_reports: Dict[str, int] = betterproto.map_field(
        7, betterproto.TYPE_STRING, betterproto.TYPE_UINT32
    )
    neighbor_cell_ids: List["NeighborCellId"] = betterproto.message_field(8)


@dataclass(eq=False, repr=False)
class ServiceModelInfo(betterproto.Message):
    oid: str = betterproto.string_field(1)
    name: str = betterproto.string_field(2)
    ran_functions: List[
        "betterproto_lib_google_protobuf.Any"
    ] = betterproto.message_field(3)
    ran_function_i_ds: List[int] = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class RcRanFunction(betterproto.Message):
    id: str = betterproto.string_field(1)
    report_styles: List["RcReportStyle"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class MhoRanFunction(betterproto.Message):
    id: str = betterproto.string_field(1)
    report_styles: List["MhoReportStyle"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class KpmRanFunction(betterproto.Message):
    id: str = betterproto.string_field(1)
    report_styles: List["KpmReportStyle"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class RsmRanFunction(betterproto.Message):
    id: str = betterproto.string_field(1)
    ric_slicing_node_capability_list: List[
        "RsmNodeSlicingCapabilityItem"
    ] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class RsmNodeSlicingCapabilityItem(betterproto.Message):
    max_number_of_slices_dl: int = betterproto.int32_field(1)
    max_number_of_slices_ul: int = betterproto.int32_field(2)
    slicing_type: "RsmSlicingType" = betterproto.enum_field(3)
    max_number_of_ues_per_slice: int = betterproto.int32_field(4)
    supported_config: List["RsmSupportedSlicingConfigItem"] = betterproto.message_field(
        5
    )


@dataclass(eq=False, repr=False)
class RsmSupportedSlicingConfigItem(betterproto.Message):
    slicing_config_type: "E2SmRsmCommand" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class RsmSliceItemList(betterproto.Message):
    rsm_slice_list: List["RsmSlicingItem"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class RsmSlicingItem(betterproto.Message):
    id: str = betterproto.string_field(1)
    slice_desc: str = betterproto.string_field(2)
    slice_parameters: "RsmSliceParameters" = betterproto.message_field(3)
    slice_type: "RsmSliceType" = betterproto.enum_field(4)
    ue_id_list: List["UeIdentity"] = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class RsmSliceParameters(betterproto.Message):
    scheduler_type: "RsmSchedulerType" = betterproto.enum_field(1)
    weight: int = betterproto.int32_field(2)
    qos_level: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class DuUeF1ApId(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class CuUeF1ApId(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class RanUeNgapId(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class EnbUeS1ApId(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class AmfUeNgapId(betterproto.Message):
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class UeIdentity(betterproto.Message):
    du_ue_f1_ap_id: "DuUeF1ApId" = betterproto.message_field(1)
    cu_ue_f1_ap_id: "CuUeF1ApId" = betterproto.message_field(2)
    ran_ue_ngap_id: "RanUeNgapId" = betterproto.message_field(3)
    enb_ue_s1_ap_id: "EnbUeS1ApId" = betterproto.message_field(4)
    amf_ue_ngap_id: "AmfUeNgapId" = betterproto.message_field(5)
    preferred_id_type: "UeIdType" = betterproto.enum_field(6)
    drb_id: "DrbId" = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class DrbId(betterproto.Message):
    four_gdrb_id: "FourGDrbId" = betterproto.message_field(1, group="drb_id")
    five_gdrb_id: "FiveGDrbId" = betterproto.message_field(2, group="drb_id")


@dataclass(eq=False, repr=False)
class FiveGDrbId(betterproto.Message):
    value: int = betterproto.int32_field(1)
    qfi: "Qfi" = betterproto.message_field(2)
    flows_map_to_drb: List["QoSflowLevelParameters"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class Qfi(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class QoSflowLevelParameters(betterproto.Message):
    dynamic_five_qi: "DynamicFiveQi" = betterproto.message_field(
        1, group="qos_flow_level_parameters"
    )
    non_dynamic_five_qi: "NonDynamicFiveQi" = betterproto.message_field(
        2, group="qos_flow_level_parameters"
    )


@dataclass(eq=False, repr=False)
class DynamicFiveQi(betterproto.Message):
    priority_level: int = betterproto.int32_field(1)
    packet_delay_budge: int = betterproto.int32_field(2)
    packet_error_rate: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class NonDynamicFiveQi(betterproto.Message):
    five_qi: "FiveQi" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class FiveQi(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class FourGDrbId(betterproto.Message):
    value: int = betterproto.int32_field(1)
    qci: "Qci" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Qci(betterproto.Message):
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class RcReportStyle(betterproto.Message):
    name: str = betterproto.string_field(1)
    type: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class KpmReportStyle(betterproto.Message):
    name: str = betterproto.string_field(1)
    type: int = betterproto.int32_field(2)
    measurements: List["KpmMeasurement"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MhoReportStyle(betterproto.Message):
    name: str = betterproto.string_field(1)
    type: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class KpmMeasurement(betterproto.Message):
    id: str = betterproto.string_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """Event is a topo operation event"""

    type: "EventType" = betterproto.enum_field(1)
    object: "Object" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CreateRequest(betterproto.Message):
    object: "Object" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CreateResponse(betterproto.Message):
    object: "Object" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class GetRequest(betterproto.Message):
    id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GetResponse(betterproto.Message):
    object: "Object" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateRequest(betterproto.Message):
    object: "Object" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateResponse(betterproto.Message):
    object: "Object" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DeleteRequest(betterproto.Message):
    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class DeleteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class Filter(betterproto.Message):
    equal: "EqualFilter" = betterproto.message_field(1, group="filter")
    not_: "NotFilter" = betterproto.message_field(2, group="filter")
    in_: "InFilter" = betterproto.message_field(3, group="filter")
    key: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class EqualFilter(betterproto.Message):
    value: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class InFilter(betterproto.Message):
    values: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class NotFilter(betterproto.Message):
    inner: "Filter" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class RelationFilter(betterproto.Message):
    """
    Filter for targets of given relation kinds and given source ids;
    optionally, filters by specified target kind
    """

    src_id: str = betterproto.string_field(1)
    relation_kind: str = betterproto.string_field(2)
    target_kind: str = betterproto.string_field(3)
    scope: "RelationFilterScope" = betterproto.enum_field(4)
    target_id: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class Filters(betterproto.Message):
    kind_filter: "Filter" = betterproto.message_field(1)
    label_filters: List["Filter"] = betterproto.message_field(2)
    relation_filter: "RelationFilter" = betterproto.message_field(3)
    object_types: List["ObjectType"] = betterproto.enum_field(4)
    with_aspects: List[str] = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class ListRequest(betterproto.Message):
    filters: "Filters" = betterproto.message_field(1)
    sort_order: "SortOrder" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ListResponse(betterproto.Message):
    objects: List["Object"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class WatchRequest(betterproto.Message):
    filters: "Filters" = betterproto.message_field(1)
    noreplay: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class WatchResponse(betterproto.Message):
    event: "Event" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Object(betterproto.Message):
    """
    Object is an one of the following: a kind (archetype of entity or
    relation), an entity, a relation
    """

    uuid: str = betterproto.string_field(9)
    id: str = betterproto.string_field(1)
    revision: int = betterproto.uint64_field(2)
    type: "ObjectType" = betterproto.enum_field(3)
    entity: "Entity" = betterproto.message_field(4, group="obj")
    relation: "Relation" = betterproto.message_field(5, group="obj")
    kind: "Kind" = betterproto.message_field(6, group="obj")
    # Map of aspects as typed values; for kind, these represent expected aspects
    # and their default values
    aspects: Dict[str, "betterproto_lib_google_protobuf.Any"] = betterproto.map_field(
        7, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    # Arbitrary labels for classification/search
    labels: Dict[str, str] = betterproto.map_field(
        8, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass(eq=False, repr=False)
class Entity(betterproto.Message):
    """Entity represents any "thing" that is represented in the topology"""

    # user-defined entity kind
    kind_id: str = betterproto.string_field(1)
    # these lists are maintained by the system and are provided as read-only
    # values for clients
    src_relation_ids: List[str] = betterproto.string_field(2)
    tgt_relation_ids: List[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class Relation(betterproto.Message):
    """
    Relation represents any "relation" between two entitites in the topology.
    """

    # user defined relation kind
    kind_id: str = betterproto.string_field(1)
    src_entity_id: str = betterproto.string_field(2)
    tgt_entity_id: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class Kind(betterproto.Message):
    """Kind represents an archetype of an object, i.e. entity or relation"""

    name: str = betterproto.string_field(1)


class TopoStub(betterproto.ServiceStub):
    async def create(self, *, object: "Object" = None) -> "CreateResponse":

        request = CreateRequest()
        if object is not None:
            request.object = object

        return await self._unary_unary(
            "/onos.topo.Topo/Create", request, CreateResponse
        )

    async def get(self, *, id: str = "") -> "GetResponse":

        request = GetRequest()
        request.id = id

        return await self._unary_unary("/onos.topo.Topo/Get", request, GetResponse)

    async def update(self, *, object: "Object" = None) -> "UpdateResponse":

        request = UpdateRequest()
        if object is not None:
            request.object = object

        return await self._unary_unary(
            "/onos.topo.Topo/Update", request, UpdateResponse
        )

    async def delete(self, *, id: str = "", revision: int = 0) -> "DeleteResponse":

        request = DeleteRequest()
        request.id = id
        request.revision = revision

        return await self._unary_unary(
            "/onos.topo.Topo/Delete", request, DeleteResponse
        )

    async def list(
        self, *, filters: "Filters" = None, sort_order: "SortOrder" = None
    ) -> "ListResponse":

        request = ListRequest()
        if filters is not None:
            request.filters = filters
        request.sort_order = sort_order

        return await self._unary_unary("/onos.topo.Topo/List", request, ListResponse)

    async def watch(
        self, *, filters: "Filters" = None, noreplay: bool = False
    ) -> AsyncIterator["WatchResponse"]:

        request = WatchRequest()
        if filters is not None:
            request.filters = filters
        request.noreplay = noreplay

        async for response in self._unary_stream(
            "/onos.topo.Topo/Watch",
            request,
            WatchResponse,
        ):
            yield response


class TopoBase(ServiceBase):
    async def create(self, object: "Object") -> "CreateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get(self, id: str) -> "GetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update(self, object: "Object") -> "UpdateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete(self, id: str, revision: int) -> "DeleteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list(self, filters: "Filters", sort_order: "SortOrder") -> "ListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch(
        self, filters: "Filters", noreplay: bool
    ) -> AsyncIterator["WatchResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "object": request.object,
        }

        response = await self.create(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "id": request.id,
        }

        response = await self.get(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_update(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "object": request.object,
        }

        response = await self.update(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "id": request.id,
            "revision": request.revision,
        }

        response = await self.delete(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "filters": request.filters,
            "sort_order": request.sort_order,
        }

        response = await self.list(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "filters": request.filters,
            "noreplay": request.noreplay,
        }

        await self._call_rpc_handler_server_stream(
            self.watch,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.topo.Topo/Create": grpclib.const.Handler(
                self.__rpc_create,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateRequest,
                CreateResponse,
            ),
            "/onos.topo.Topo/Get": grpclib.const.Handler(
                self.__rpc_get,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetRequest,
                GetResponse,
            ),
            "/onos.topo.Topo/Update": grpclib.const.Handler(
                self.__rpc_update,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpdateRequest,
                UpdateResponse,
            ),
            "/onos.topo.Topo/Delete": grpclib.const.Handler(
                self.__rpc_delete,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteRequest,
                DeleteResponse,
            ),
            "/onos.topo.Topo/List": grpclib.const.Handler(
                self.__rpc_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListRequest,
                ListResponse,
            ),
            "/onos.topo.Topo/Watch": grpclib.const.Handler(
                self.__rpc_watch,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchRequest,
                WatchResponse,
            ),
        }


import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/ransim/model/model.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict, List, Optional

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class EventType(betterproto.Enum):
    """Change event type"""

    # NONE indicates this response represents a pre-existing entity
    NONE = 0
    # CREATED indicates a new entity was created
    CREATED = 1
    # UPDATED indicates an existing entity was updated
    UPDATED = 2
    # DELETED indicates an entity was deleted
    DELETED = 3


@dataclass(eq=False, repr=False)
class DataSet(betterproto.Message):
    type: str = betterproto.string_field(1)
    data: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class LoadRequest(betterproto.Message):
    data_set: List["DataSet"] = betterproto.message_field(1)
    resume: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class LoadResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ClearRequest(betterproto.Message):
    resume: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class ClearResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class PlmnIdRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class PlmnIdResponse(betterproto.Message):
    plmnid: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CreateNodeRequest(betterproto.Message):
    """CreateNodeRequest create a node request"""

    node: "_types__.Node" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CreateNodeResponse(betterproto.Message):
    """CreateNodeResponse create a node response"""

    pass


@dataclass(eq=False, repr=False)
class GetNodeRequest(betterproto.Message):
    """GetNodeRequest get a node request"""

    enbid: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class GetNodeResponse(betterproto.Message):
    """GetNodeResponse get a node response"""

    node: "_types__.Node" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateNodeRequest(betterproto.Message):
    """UpdateNodeRequest update a node request"""

    node: "_types__.Node" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateNodeResponse(betterproto.Message):
    """UpdateNodeResponse update a node response"""

    pass


@dataclass(eq=False, repr=False)
class DeleteNodeRequest(betterproto.Message):
    """DeleteNodeRequest delete a node request"""

    enbid: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class DeleteNodeResponse(betterproto.Message):
    """DeleteNodeResponse delete a node response"""

    pass


@dataclass(eq=False, repr=False)
class ListNodesRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListNodesResponse(betterproto.Message):
    node: "_types__.Node" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class WatchNodesRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class WatchNodesResponse(betterproto.Message):
    node: "_types__.Node" = betterproto.message_field(1)
    type: "EventType" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class AgentControlRequest(betterproto.Message):
    enbid: int = betterproto.uint64_field(1)
    command: str = betterproto.string_field(2)
    args: List[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class AgentControlResponse(betterproto.Message):
    node: "_types__.Node" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CreateCellRequest(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CreateCellResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetCellRequest(betterproto.Message):
    ecgi: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class GetCellResponse(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateCellRequest(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class UpdateCellResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DeleteCellRequest(betterproto.Message):
    enbid: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class DeleteCellResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class WatchCellsRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class WatchCellsResponse(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)
    type: "EventType" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ListCellsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListCellsResponse(betterproto.Message):
    cell: "_types__.Cell" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CreateRouteRequest(betterproto.Message):
    route: "_types__.Route" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CreateRouteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetRouteRequest(betterproto.Message):
    imsi: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class GetRouteResponse(betterproto.Message):
    route: "_types__.Route" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DeleteRouteRequest(betterproto.Message):
    enbid: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class DeleteRouteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class WatchRoutesRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class WatchRoutesResponse(betterproto.Message):
    route: "_types__.Route" = betterproto.message_field(1)
    type: "EventType" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ListRoutesRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListRoutesResponse(betterproto.Message):
    route: "_types__.Route" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class GetUeRequest(betterproto.Message):
    imsi: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class GetUeResponse(betterproto.Message):
    ue: "_types__.Ue" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MoveToCellRequest(betterproto.Message):
    imsi: int = betterproto.uint32_field(1)
    ecgi: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class MoveToCellResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class MoveToLocationRequest(betterproto.Message):
    imsi: int = betterproto.uint32_field(1)
    location: "_types__.Point" = betterproto.message_field(2)
    heading: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class MoveToLocationResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DeleteUeRequest(betterproto.Message):
    imsi: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class DeleteUeResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class WatchUEsRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)
    no_subscribe: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class WatchUEsResponse(betterproto.Message):
    ue: "_types__.Ue" = betterproto.message_field(1)
    type: "EventType" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ListUEsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListUEsResponse(betterproto.Message):
    ue: "_types__.Ue" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class GetUeCountRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetUeCountResponse(betterproto.Message):
    count: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class SetUeCountRequest(betterproto.Message):
    count: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class SetUeCountResponse(betterproto.Message):
    pass


class ModelServiceStub(betterproto.ServiceStub):
    async def load(
        self, *, data_set: Optional[List["DataSet"]] = None, resume: bool = False
    ) -> "LoadResponse":
        data_set = data_set or []

        request = LoadRequest()
        if data_set is not None:
            request.data_set = data_set
        request.resume = resume

        return await self._unary_unary(
            "/onos.ransim.model.ModelService/Load", request, LoadResponse
        )

    async def clear(self, *, resume: bool = False) -> "ClearResponse":

        request = ClearRequest()
        request.resume = resume

        return await self._unary_unary(
            "/onos.ransim.model.ModelService/Clear", request, ClearResponse
        )


class NodeModelStub(betterproto.ServiceStub):
    async def get_plmn_id(self) -> "PlmnIdResponse":

        request = PlmnIdRequest()

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/GetPlmnID", request, PlmnIdResponse
        )

    async def create_node(
        self, *, node: "_types__.Node" = None
    ) -> "CreateNodeResponse":

        request = CreateNodeRequest()
        if node is not None:
            request.node = node

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/CreateNode", request, CreateNodeResponse
        )

    async def get_node(self, *, enbid: int = 0) -> "GetNodeResponse":

        request = GetNodeRequest()
        request.enbid = enbid

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/GetNode", request, GetNodeResponse
        )

    async def update_node(
        self, *, node: "_types__.Node" = None
    ) -> "UpdateNodeResponse":

        request = UpdateNodeRequest()
        if node is not None:
            request.node = node

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/UpdateNode", request, UpdateNodeResponse
        )

    async def delete_node(self, *, enbid: int = 0) -> "DeleteNodeResponse":

        request = DeleteNodeRequest()
        request.enbid = enbid

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/DeleteNode", request, DeleteNodeResponse
        )

    async def watch_nodes(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["WatchNodesResponse"]:

        request = WatchNodesRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.model.NodeModel/WatchNodes",
            request,
            WatchNodesResponse,
        ):
            yield response

    async def list_nodes(self) -> AsyncIterator["ListNodesResponse"]:

        request = ListNodesRequest()

        async for response in self._unary_stream(
            "/onos.ransim.model.NodeModel/ListNodes",
            request,
            ListNodesResponse,
        ):
            yield response

    async def agent_control(
        self, *, enbid: int = 0, command: str = "", args: Optional[List[str]] = None
    ) -> "AgentControlResponse":
        args = args or []

        request = AgentControlRequest()
        request.enbid = enbid
        request.command = command
        request.args = args

        return await self._unary_unary(
            "/onos.ransim.model.NodeModel/AgentControl", request, AgentControlResponse
        )


class CellModelStub(betterproto.ServiceStub):
    async def create_cell(
        self, *, cell: "_types__.Cell" = None
    ) -> "CreateCellResponse":

        request = CreateCellRequest()
        if cell is not None:
            request.cell = cell

        return await self._unary_unary(
            "/onos.ransim.model.CellModel/CreateCell", request, CreateCellResponse
        )

    async def delete_cell(self, *, enbid: int = 0) -> "DeleteCellResponse":

        request = DeleteCellRequest()
        request.enbid = enbid

        return await self._unary_unary(
            "/onos.ransim.model.CellModel/DeleteCell", request, DeleteCellResponse
        )

    async def update_cell(
        self, *, cell: "_types__.Cell" = None
    ) -> "UpdateCellResponse":

        request = UpdateCellRequest()
        if cell is not None:
            request.cell = cell

        return await self._unary_unary(
            "/onos.ransim.model.CellModel/UpdateCell", request, UpdateCellResponse
        )

    async def get_cell(self, *, ecgi: int = 0) -> "GetCellResponse":

        request = GetCellRequest()
        request.ecgi = ecgi

        return await self._unary_unary(
            "/onos.ransim.model.CellModel/GetCell", request, GetCellResponse
        )

    async def watch_cells(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["WatchCellsResponse"]:

        request = WatchCellsRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.model.CellModel/WatchCells",
            request,
            WatchCellsResponse,
        ):
            yield response

    async def list_cells(self) -> AsyncIterator["ListCellsResponse"]:

        request = ListCellsRequest()

        async for response in self._unary_stream(
            "/onos.ransim.model.CellModel/ListCells",
            request,
            ListCellsResponse,
        ):
            yield response


class RouteModelStub(betterproto.ServiceStub):
    async def create_route(
        self, *, route: "_types__.Route" = None
    ) -> "CreateRouteResponse":

        request = CreateRouteRequest()
        if route is not None:
            request.route = route

        return await self._unary_unary(
            "/onos.ransim.model.RouteModel/CreateRoute", request, CreateRouteResponse
        )

    async def delete_route(self, *, enbid: int = 0) -> "DeleteRouteResponse":

        request = DeleteRouteRequest()
        request.enbid = enbid

        return await self._unary_unary(
            "/onos.ransim.model.RouteModel/DeleteRoute", request, DeleteRouteResponse
        )

    async def get_route(self, *, imsi: int = 0) -> "GetRouteResponse":

        request = GetRouteRequest()
        request.imsi = imsi

        return await self._unary_unary(
            "/onos.ransim.model.RouteModel/GetRoute", request, GetRouteResponse
        )

    async def watch_routes(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["WatchRoutesResponse"]:

        request = WatchRoutesRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.model.RouteModel/WatchRoutes",
            request,
            WatchRoutesResponse,
        ):
            yield response

    async def list_routes(self) -> AsyncIterator["ListRoutesResponse"]:

        request = ListRoutesRequest()

        async for response in self._unary_stream(
            "/onos.ransim.model.RouteModel/ListRoutes",
            request,
            ListRoutesResponse,
        ):
            yield response


class UeModelStub(betterproto.ServiceStub):
    async def get_ue(self) -> "GetUeResponse":

        request = GetUeRequest()

        return await self._unary_unary(
            "/onos.ransim.model.UEModel/GetUE", request, GetUeResponse
        )

    async def move_to_cell(
        self, *, imsi: int = 0, ecgi: int = 0
    ) -> "MoveToCellResponse":

        request = MoveToCellRequest()
        request.imsi = imsi
        request.ecgi = ecgi

        return await self._unary_unary(
            "/onos.ransim.model.UEModel/MoveToCell", request, MoveToCellResponse
        )

    async def move_to_location(
        self, *, imsi: int = 0, location: "_types__.Point" = None, heading: int = 0
    ) -> "MoveToLocationResponse":

        request = MoveToLocationRequest()
        request.imsi = imsi
        if location is not None:
            request.location = location
        request.heading = heading

        return await self._unary_unary(
            "/onos.ransim.model.UEModel/MoveToLocation", request, MoveToLocationResponse
        )

    async def delete_ue(self) -> "DeleteUeResponse":

        request = DeleteUeRequest()

        return await self._unary_unary(
            "/onos.ransim.model.UEModel/DeleteUE", request, DeleteUeResponse
        )

    async def watch_u_es(
        self, *, no_replay: bool = False, no_subscribe: bool = False
    ) -> AsyncIterator["WatchUEsResponse"]:

        request = WatchUEsRequest()
        request.no_replay = no_replay
        request.no_subscribe = no_subscribe

        async for response in self._unary_stream(
            "/onos.ransim.model.UEModel/WatchUEs",
            request,
            WatchUEsResponse,
        ):
            yield response

    async def list_u_es(self) -> AsyncIterator["ListUEsResponse"]:

        request = ListUEsRequest()

        async for response in self._unary_stream(
            "/onos.ransim.model.UEModel/ListUEs",
            request,
            ListUEsResponse,
        ):
            yield response

    async def get_ue_count(self) -> "GetUeCountResponse":

        request = GetUeCountRequest()

        return await self._unary_unary(
            "/onos.ransim.model.UEModel/GetUECount", request, GetUeCountResponse
        )

    async def set_ue_count(self) -> "SetUeCountResponse":

        request = SetUeCountRequest()

        return await self._unary_unary(
            "/onos.ransim.model.UEModel/SetUECount", request, SetUeCountResponse
        )


class ModelServiceBase(ServiceBase):
    async def load(
        self, data_set: Optional[List["DataSet"]], resume: bool
    ) -> "LoadResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def clear(self, resume: bool) -> "ClearResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_load(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "data_set": request.data_set,
            "resume": request.resume,
        }

        response = await self.load(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_clear(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "resume": request.resume,
        }

        response = await self.clear(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.ransim.model.ModelService/Load": grpclib.const.Handler(
                self.__rpc_load,
                grpclib.const.Cardinality.UNARY_UNARY,
                LoadRequest,
                LoadResponse,
            ),
            "/onos.ransim.model.ModelService/Clear": grpclib.const.Handler(
                self.__rpc_clear,
                grpclib.const.Cardinality.UNARY_UNARY,
                ClearRequest,
                ClearResponse,
            ),
        }


class NodeModelBase(ServiceBase):
    async def get_plmn_id(self) -> "PlmnIdResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def create_node(self, node: "_types__.Node") -> "CreateNodeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_node(self, enbid: int) -> "GetNodeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update_node(self, node: "_types__.Node") -> "UpdateNodeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_node(self, enbid: int) -> "DeleteNodeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_nodes(
        self, no_replay: bool, no_subscribe: bool
    ) -> AsyncIterator["WatchNodesResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_nodes(self) -> AsyncIterator["ListNodesResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def agent_control(
        self, enbid: int, command: str, args: Optional[List[str]]
    ) -> "AgentControlResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_plmn_id(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_plmn_id(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_create_node(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "node": request.node,
        }

        response = await self.create_node(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_node(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "enbid": request.enbid,
        }

        response = await self.get_node(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_update_node(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "node": request.node,
        }

        response = await self.update_node(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_node(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "enbid": request.enbid,
        }

        response = await self.delete_node(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch_nodes(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "no_replay": request.no_replay,
            "no_subscribe": request.no_subscribe,
        }

        await self._call_rpc_handler_server_stream(
            self.watch_nodes,
            stream,
            request_kwargs,
        )

    async def __rpc_list_nodes(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.list_nodes,
            stream,
            request_kwargs,
        )

    async def __rpc_agent_control(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "enbid": request.enbid,
            "command": request.command,
            "args": request.args,
        }

        response = await self.agent_control(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.ransim.model.NodeModel/GetPlmnID": grpclib.const.Handler(
                self.__rpc_get_plmn_id,
                grpclib.const.Cardinality.UNARY_UNARY,
                PlmnIdRequest,
                PlmnIdResponse,
            ),
            "/onos.ransim.model.NodeModel/CreateNode": grpclib.const.Handler(
                self.__rpc_create_node,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateNodeRequest,
                CreateNodeResponse,
            ),
            "/onos.ransim.model.NodeModel/GetNode": grpclib.const.Handler(
                self.__rpc_get_node,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetNodeRequest,
                GetNodeResponse,
            ),
            "/onos.ransim.model.NodeModel/UpdateNode": grpclib.const.Handler(
                self.__rpc_update_node,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpdateNodeRequest,
                UpdateNodeResponse,
            ),
            "/onos.ransim.model.NodeModel/DeleteNode": grpclib.const.Handler(
                self.__rpc_delete_node,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteNodeRequest,
                DeleteNodeResponse,
            ),
            "/onos.ransim.model.NodeModel/WatchNodes": grpclib.const.Handler(
                self.__rpc_watch_nodes,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchNodesRequest,
                WatchNodesResponse,
            ),
            "/onos.ransim.model.NodeModel/ListNodes": grpclib.const.Handler(
                self.__rpc_list_nodes,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListNodesRequest,
                ListNodesResponse,
            ),
            "/onos.ransim.model.NodeModel/AgentControl": grpclib.const.Handler(
                self.__rpc_agent_control,
                grpclib.const.Cardinality.UNARY_UNARY,
                AgentControlRequest,
                AgentControlResponse,
            ),
        }


class CellModelBase(ServiceBase):
    async def create_cell(self, cell: "_types__.Cell") -> "CreateCellResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_cell(self, enbid: int) -> "DeleteCellResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def update_cell(self, cell: "_types__.Cell") -> "UpdateCellResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_cell(self, ecgi: int) -> "GetCellResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_cells(
        self, no_replay: bool, no_subscribe: bool
    ) -> AsyncIterator["WatchCellsResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_cells(self) -> AsyncIterator["ListCellsResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create_cell(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "cell": request.cell,
        }

        response = await self.create_cell(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_cell(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "enbid": request.enbid,
        }

        response = await self.delete_cell(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_update_cell(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "cell": request.cell,
        }

        response = await self.update_cell(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_cell(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "ecgi": request.ecgi,
        }

        response = await self.get_cell(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch_cells(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "no_replay": request.no_replay,
            "no_subscribe": request.no_subscribe,
        }

        await self._call_rpc_handler_server_stream(
            self.watch_cells,
            stream,
            request_kwargs,
        )

    async def __rpc_list_cells(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.list_cells,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.ransim.model.CellModel/CreateCell": grpclib.const.Handler(
                self.__rpc_create_cell,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateCellRequest,
                CreateCellResponse,
            ),
            "/onos.ransim.model.CellModel/DeleteCell": grpclib.const.Handler(
                self.__rpc_delete_cell,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteCellRequest,
                DeleteCellResponse,
            ),
            "/onos.ransim.model.CellModel/UpdateCell": grpclib.const.Handler(
                self.__rpc_update_cell,
                grpclib.const.Cardinality.UNARY_UNARY,
                UpdateCellRequest,
                UpdateCellResponse,
            ),
            "/onos.ransim.model.CellModel/GetCell": grpclib.const.Handler(
                self.__rpc_get_cell,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetCellRequest,
                GetCellResponse,
            ),
            "/onos.ransim.model.CellModel/WatchCells": grpclib.const.Handler(
                self.__rpc_watch_cells,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchCellsRequest,
                WatchCellsResponse,
            ),
            "/onos.ransim.model.CellModel/ListCells": grpclib.const.Handler(
                self.__rpc_list_cells,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListCellsRequest,
                ListCellsResponse,
            ),
        }


class RouteModelBase(ServiceBase):
    async def create_route(self, route: "_types__.Route") -> "CreateRouteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_route(self, enbid: int) -> "DeleteRouteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_route(self, imsi: int) -> "GetRouteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_routes(
        self, no_replay: bool, no_subscribe: bool
    ) -> AsyncIterator["WatchRoutesResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_routes(self) -> AsyncIterator["ListRoutesResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create_route(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "route": request.route,
        }

        response = await self.create_route(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_route(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "enbid": request.enbid,
        }

        response = await self.delete_route(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_route(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "imsi": request.imsi,
        }

        response = await self.get_route(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch_routes(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "no_replay": request.no_replay,
            "no_subscribe": request.no_subscribe,
        }

        await self._call_rpc_handler_server_stream(
            self.watch_routes,
            stream,
            request_kwargs,
        )

    async def __rpc_list_routes(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.list_routes,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.ransim.model.RouteModel/CreateRoute": grpclib.const.Handler(
                self.__rpc_create_route,
                grpclib.const.Cardinality.UNARY_UNARY,
                CreateRouteRequest,
                CreateRouteResponse,
            ),
            "/onos.ransim.model.RouteModel/DeleteRoute": grpclib.const.Handler(
                self.__rpc_delete_route,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteRouteRequest,
                DeleteRouteResponse,
            ),
            "/onos.ransim.model.RouteModel/GetRoute": grpclib.const.Handler(
                self.__rpc_get_route,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetRouteRequest,
                GetRouteResponse,
            ),
            "/onos.ransim.model.RouteModel/WatchRoutes": grpclib.const.Handler(
                self.__rpc_watch_routes,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchRoutesRequest,
                WatchRoutesResponse,
            ),
            "/onos.ransim.model.RouteModel/ListRoutes": grpclib.const.Handler(
                self.__rpc_list_routes,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListRoutesRequest,
                ListRoutesResponse,
            ),
        }


class UeModelBase(ServiceBase):
    async def get_ue(self) -> "GetUeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def move_to_cell(self, imsi: int, ecgi: int) -> "MoveToCellResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def move_to_location(
        self, imsi: int, location: "_types__.Point", heading: int
    ) -> "MoveToLocationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete_ue(self) -> "DeleteUeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_u_es(
        self, no_replay: bool, no_subscribe: bool
    ) -> AsyncIterator["WatchUEsResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_u_es(self) -> AsyncIterator["ListUEsResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_ue_count(self) -> "GetUeCountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_ue_count(self) -> "SetUeCountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_ue(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_ue(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_move_to_cell(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "imsi": request.imsi,
            "ecgi": request.ecgi,
        }

        response = await self.move_to_cell(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_move_to_location(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "imsi": request.imsi,
            "location": request.location,
            "heading": request.heading,
        }

        response = await self.move_to_location(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete_ue(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.delete_ue(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch_u_es(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "no_replay": request.no_replay,
            "no_subscribe": request.no_subscribe,
        }

        await self._call_rpc_handler_server_stream(
            self.watch_u_es,
            stream,
            request_kwargs,
        )

    async def __rpc_list_u_es(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        await self._call_rpc_handler_server_stream(
            self.list_u_es,
            stream,
            request_kwargs,
        )

    async def __rpc_get_ue_count(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_ue_count(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_set_ue_count(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.set_ue_count(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.ransim.model.UEModel/GetUE": grpclib.const.Handler(
                self.__rpc_get_ue,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetUeRequest,
                GetUeResponse,
            ),
            "/onos.ransim.model.UEModel/MoveToCell": grpclib.const.Handler(
                self.__rpc_move_to_cell,
                grpclib.const.Cardinality.UNARY_UNARY,
                MoveToCellRequest,
                MoveToCellResponse,
            ),
            "/onos.ransim.model.UEModel/MoveToLocation": grpclib.const.Handler(
                self.__rpc_move_to_location,
                grpclib.const.Cardinality.UNARY_UNARY,
                MoveToLocationRequest,
                MoveToLocationResponse,
            ),
            "/onos.ransim.model.UEModel/DeleteUE": grpclib.const.Handler(
                self.__rpc_delete_ue,
                grpclib.const.Cardinality.UNARY_UNARY,
                DeleteUeRequest,
                DeleteUeResponse,
            ),
            "/onos.ransim.model.UEModel/WatchUEs": grpclib.const.Handler(
                self.__rpc_watch_u_es,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchUEsRequest,
                WatchUEsResponse,
            ),
            "/onos.ransim.model.UEModel/ListUEs": grpclib.const.Handler(
                self.__rpc_list_u_es,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListUEsRequest,
                ListUEsResponse,
            ),
            "/onos.ransim.model.UEModel/GetUECount": grpclib.const.Handler(
                self.__rpc_get_ue_count,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetUeCountRequest,
                GetUeCountResponse,
            ),
            "/onos.ransim.model.UEModel/SetUECount": grpclib.const.Handler(
                self.__rpc_set_ue_count,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetUeCountRequest,
                SetUeCountResponse,
            ),
        }


from .. import types as _types__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/v2/configuration/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase


class State(betterproto.Enum):
    """State is the state of a phase"""

    # PENDING indicates the phase is pending
    PENDING = 0
    # COMPLETE indicates the phase is complete
    COMPLETE = 2
    # FAILED indicates the phase failed
    FAILED = 3


@dataclass(eq=False, repr=False)
class Configuration(betterproto.Message):
    """Configuration represents complete desired target configuration"""

    # 'id' is a unique configuration identifier
    id: str = betterproto.string_field(1)
    # 'target_id' is the target to which the desired target configuration applies
    target_id: str = betterproto.string_field(2)
    # 'target_version' is the version to which desired target configuration
    # applies
    target_version: str = betterproto.string_field(3)
    # 'target_type' is an optional target type to which to apply this desired
    # target configuration
    target_type: str = betterproto.string_field(4)
    # 'values' is a list of path/values to set
    values: List["__v2__.PathValue"] = betterproto.message_field(7)


from ... import v2 as __v2__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/e2t/e2/v1beta1/control.proto, onos/e2t/e2/v1beta1/e2.proto, onos/e2t/e2/v1beta1/subscription.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import AsyncIterator, Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class Encoding(betterproto.Enum):
    PROTO = 0
    ASN1_PER = 1
    ASN1_XER = 2


class ErrorCauseMiscType(betterproto.Enum):
    UNSPECIFIED = 0
    CONTROL_PROCESSING_OVERLOAD = 1
    HARDWARE_FAILURE = 2
    OM_INTERVENTION = 3


class ErrorCauseProtocolType(betterproto.Enum):
    UNSPECIFIED = 0
    TRANSFER_SYNTAX_ERROR = 1
    ABSTRACT_SYNTAX_ERROR_REJECT = 2
    ABSTRACT_SYNTAX_ERROR_IGNORE_AND_NOTIFY = 3
    MESSAGE_NOT_COMPATIBLE_WITH_RECEIVER_STATE = 4
    SEMANTIC_ERROR = 5
    ABSTRACT_SYNTAX_ERROR_FALSELY_CONSTRUCTED_MESSAGE = 6


class ErrorCauseRicType(betterproto.Enum):
    UNSPECIFIED = 0
    RAN_FUNCTION_ID_INVALID = 1
    ACTION_NOT_SUPPORTED = 2
    EXCESSIVE_ACTIONS = 3
    DUPLICATE_ACTION = 4
    DUPLICATE_EVENT = 5
    FUNCTION_RESOURCE_LIMIT = 6
    REQUEST_ID_UNKNOWN = 7
    INCONSISTENT_ACTION_SUBSEQUENT_ACTION_SEQUENCE = 8
    CONTROL_MESSAGE_INVALID = 9
    CALL_PROCESS_ID_INVALID = 10


class ErrorCauseRicServiceType(betterproto.Enum):
    UNSPECIFIED = 0
    FUNCTION_NOT_REQUIRED = 1
    EXCESSIVE_FUNCTIONS = 2
    RIC_RESOURCE_LIMIT = 3


class ErrorCauseTransportType(betterproto.Enum):
    UNSPECIFIED = 0
    TRANSPORT_RESOURCE_UNAVAILABLE = 1


class ActionType(betterproto.Enum):
    ACTION_TYPE_REPORT = 0
    ACTION_TYPE_INSERT = 1
    ACTION_TYPE_POLICY = 2


class SubsequentActionType(betterproto.Enum):
    SUBSEQUENT_ACTION_TYPE_CONTINUE = 0
    SUBSEQUENT_ACTION_TYPE_WAIT = 1


class TimeToWait(betterproto.Enum):
    TIME_TO_WAIT_ZERO = 0
    TIME_TO_WAIT_W1MS = 1
    TIME_TO_WAIT_W2MS = 2
    TIME_TO_WAIT_W5MS = 3
    TIME_TO_WAIT_W10MS = 4
    TIME_TO_WAIT_W20MS = 5
    TIME_TO_WAIT_W30MS = 6
    TIME_TO_WAIT_W40MS = 7
    TIME_TO_WAIT_W50MS = 8
    TIME_TO_WAIT_W100MS = 9
    TIME_TO_WAIT_W200MS = 10
    TIME_TO_WAIT_W500MS = 11
    TIME_TO_WAIT_W1S = 12
    TIME_TO_WAIT_W2S = 13
    TIME_TO_WAIT_W5S = 14
    TIME_TO_WAIT_W10S = 15
    TIME_TO_WAIT_W20S = 16
    TIME_TO_WAIT_W60S = 17


class ChannelEventType(betterproto.Enum):
    CHANNEL_EVENT_UNKNOWN = 0
    CHANNEL_CREATED = 1
    CHANNEL_UPDATED = 2
    CHANNEL_DELETED = 3
    CHANNEL_REPLAYED = 4


class SubscriptionEventType(betterproto.Enum):
    SUBSCRIPTION_EVENT_UNKNOWN = 0
    SUBSCRIPTION_CREATED = 1
    SUBSCRIPTION_UPDATED = 2
    SUBSCRIPTION_DELETED = 3
    SUBSCRIPTION_REPLAYED = 4


class ChannelPhase(betterproto.Enum):
    CHANNEL_CLOSED = 0
    CHANNEL_OPEN = 1


class ChannelState(betterproto.Enum):
    # CHANNEL_PENDING indicates the channel phase is pending
    CHANNEL_PENDING = 0
    # CHANNEL_COMPLETE indicates the channel phase is complete
    CHANNEL_COMPLETE = 1
    # CHANNEL_FAILED indicates the channel phase failed
    CHANNEL_FAILED = 2


class SubscriptionPhase(betterproto.Enum):
    SUBSCRIPTION_CLOSED = 0
    SUBSCRIPTION_OPEN = 1


class SubscriptionState(betterproto.Enum):
    # SUBSCRIPTION_PENDING indicates the subscription phase is pending
    SUBSCRIPTION_PENDING = 0
    # SUBSCRIPTION_COMPLETE indicates the subscription phase is complete
    SUBSCRIPTION_COMPLETE = 1
    # SUBSCRIPTION_FAILED indicates the subscription phase failed
    SUBSCRIPTION_FAILED = 2


@dataclass(eq=False, repr=False)
class RequestHeaders(betterproto.Message):
    app_id: str = betterproto.string_field(1)
    app_instance_id: str = betterproto.string_field(2)
    e2_node_id: str = betterproto.string_field(3)
    service_model: "ServiceModel" = betterproto.message_field(4)
    encoding: "Encoding" = betterproto.enum_field(5)


@dataclass(eq=False, repr=False)
class ResponseHeaders(betterproto.Message):
    encoding: "Encoding" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class ServiceModel(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class Error(betterproto.Message):
    """Error is an E2AP protocol error"""

    cause: "ErrorCause" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ErrorCause(betterproto.Message):
    unknown: "ErrorCauseUnknown" = betterproto.message_field(1, group="cause")
    protocol: "ErrorCauseProtocol" = betterproto.message_field(2, group="cause")
    ric: "ErrorCauseRic" = betterproto.message_field(3, group="cause")
    ric_service: "ErrorCauseRicService" = betterproto.message_field(4, group="cause")
    transport: "ErrorCauseTransport" = betterproto.message_field(5, group="cause")
    misc: "ErrorCauseMisc" = betterproto.message_field(6, group="cause")


@dataclass(eq=False, repr=False)
class ErrorCauseUnknown(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ErrorCauseMisc(betterproto.Message):
    type: "ErrorCauseMiscType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class ErrorCauseProtocol(betterproto.Message):
    type: "ErrorCauseProtocolType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class ErrorCauseRic(betterproto.Message):
    type: "ErrorCauseRicType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class ErrorCauseRicService(betterproto.Message):
    type: "ErrorCauseRicServiceType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class ErrorCauseTransport(betterproto.Message):
    type: "ErrorCauseTransportType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class ControlRequest(betterproto.Message):
    headers: "RequestHeaders" = betterproto.message_field(1)
    message: "ControlMessage" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ControlMessage(betterproto.Message):
    header: bytes = betterproto.bytes_field(1)
    payload: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class ControlResponse(betterproto.Message):
    headers: "ResponseHeaders" = betterproto.message_field(1)
    outcome: "ControlOutcome" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ControlOutcome(betterproto.Message):
    payload: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class SubscribeRequest(betterproto.Message):
    headers: "RequestHeaders" = betterproto.message_field(1)
    transaction_id: str = betterproto.string_field(2)
    subscription: "SubscriptionSpec" = betterproto.message_field(3)
    transaction_timeout: timedelta = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class SubscribeResponse(betterproto.Message):
    headers: "ResponseHeaders" = betterproto.message_field(1)
    ack: "Acknowledgement" = betterproto.message_field(2, group="message")
    indication: "Indication" = betterproto.message_field(3, group="message")


@dataclass(eq=False, repr=False)
class UnsubscribeRequest(betterproto.Message):
    headers: "RequestHeaders" = betterproto.message_field(1)
    transaction_id: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class UnsubscribeResponse(betterproto.Message):
    headers: "ResponseHeaders" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SubscriptionSpec(betterproto.Message):
    event_trigger: "EventTrigger" = betterproto.message_field(1)
    actions: List["Action"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class EventTrigger(betterproto.Message):
    payload: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class Action(betterproto.Message):
    id: int = betterproto.int32_field(1)
    type: "ActionType" = betterproto.enum_field(2)
    payload: bytes = betterproto.bytes_field(3)
    subsequent_action: "SubsequentAction" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class SubsequentAction(betterproto.Message):
    type: "SubsequentActionType" = betterproto.enum_field(1)
    time_to_wait: "TimeToWait" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class Acknowledgement(betterproto.Message):
    channel_id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class Indication(betterproto.Message):
    header: bytes = betterproto.bytes_field(1)
    payload: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class GetChannelRequest(betterproto.Message):
    channel_id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GetChannelResponse(betterproto.Message):
    channel: "Channel" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ListChannelsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListChannelsResponse(betterproto.Message):
    channels: List["Channel"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class WatchChannelsRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class WatchChannelsResponse(betterproto.Message):
    event: "ChannelEvent" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ChannelEvent(betterproto.Message):
    type: "ChannelEventType" = betterproto.enum_field(1)
    channel: "Channel" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GetSubscriptionRequest(betterproto.Message):
    subscription_id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class GetSubscriptionResponse(betterproto.Message):
    subscription: "Subscription" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ListSubscriptionsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ListSubscriptionsResponse(betterproto.Message):
    subscriptions: List["Subscription"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class WatchSubscriptionsRequest(betterproto.Message):
    no_replay: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class WatchSubscriptionsResponse(betterproto.Message):
    event: "SubscriptionEvent" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SubscriptionEvent(betterproto.Message):
    type: "SubscriptionEventType" = betterproto.enum_field(1)
    subscription: "Subscription" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ChannelMeta(betterproto.Message):
    """ChannelMeta contains metadata relating to a subscription channel"""

    # app_id is an identifier for the application that created the channel shared
    # across all instances of the application
    app_id: str = betterproto.string_field(1)
    # app_instance_id is an identifier unique to a single node within the
    # application that created the channel
    app_instance_id: str = betterproto.string_field(2)
    # e2_node_id is the identifier for the E2 node to which the subscription is
    # targeted
    e2_node_id: str = betterproto.string_field(3)
    # transaction_id is a persistent identifier for the transaction
    transaction_id: str = betterproto.string_field(4)
    # subscription_id is the identifier for the underlying subscription
    subscription_id: str = betterproto.string_field(5)
    # service_model is the service model specification
    service_model: "ServiceModel" = betterproto.message_field(6)
    # encoding indicates the type of encoding used within the parent message
    encoding: "Encoding" = betterproto.enum_field(7)
    revision: int = betterproto.uint64_field(8)
    finalizers: List[str] = betterproto.string_field(9)


@dataclass(eq=False, repr=False)
class Channel(betterproto.Message):
    id: str = betterproto.string_field(1)
    meta: "ChannelMeta" = betterproto.message_field(2)
    spec: "ChannelSpec" = betterproto.message_field(3)
    status: "ChannelStatus" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class ChannelSpec(betterproto.Message):
    subscription: "SubscriptionSpec" = betterproto.message_field(1)
    transaction_timeout: timedelta = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ChannelStatus(betterproto.Message):
    phase: "ChannelPhase" = betterproto.enum_field(1)
    state: "ChannelState" = betterproto.enum_field(2)
    error: "Error" = betterproto.message_field(3)
    timestamp: datetime = betterproto.message_field(4)
    term: int = betterproto.uint64_field(5)
    master: str = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class SubscriptionMeta(betterproto.Message):
    e2_node_id: str = betterproto.string_field(1)
    service_model: "ServiceModel" = betterproto.message_field(2)
    encoding: "Encoding" = betterproto.enum_field(3)
    revision: int = betterproto.uint64_field(4)
    finalizers: List[str] = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class Subscription(betterproto.Message):
    id: str = betterproto.string_field(1)
    meta: "SubscriptionMeta" = betterproto.message_field(2)
    spec: "SubscriptionSpec" = betterproto.message_field(3)
    status: "SubscriptionStatus" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class SubscriptionStatus(betterproto.Message):
    phase: "SubscriptionPhase" = betterproto.enum_field(1)
    state: "SubscriptionState" = betterproto.enum_field(2)
    error: "Error" = betterproto.message_field(3)
    channels: List[str] = betterproto.string_field(4)
    term: int = betterproto.uint64_field(5)
    master: str = betterproto.string_field(6)


class ControlServiceStub(betterproto.ServiceStub):
    async def control(
        self, *, headers: "RequestHeaders" = None, message: "ControlMessage" = None
    ) -> "ControlResponse":

        request = ControlRequest()
        if headers is not None:
            request.headers = headers
        if message is not None:
            request.message = message

        return await self._unary_unary(
            "/onos.e2t.e2.v1beta1.ControlService/Control", request, ControlResponse
        )


class SubscriptionServiceStub(betterproto.ServiceStub):
    async def subscribe(
        self,
        *,
        headers: "RequestHeaders" = None,
        transaction_id: str = "",
        subscription: "SubscriptionSpec" = None,
        transaction_timeout: timedelta = None,
    ) -> AsyncIterator["SubscribeResponse"]:

        request = SubscribeRequest()
        if headers is not None:
            request.headers = headers
        request.transaction_id = transaction_id
        if subscription is not None:
            request.subscription = subscription
        if transaction_timeout is not None:
            request.transaction_timeout = transaction_timeout

        async for response in self._unary_stream(
            "/onos.e2t.e2.v1beta1.SubscriptionService/Subscribe",
            request,
            SubscribeResponse,
        ):
            yield response

    async def unsubscribe(
        self, *, headers: "RequestHeaders" = None, transaction_id: str = ""
    ) -> "UnsubscribeResponse":

        request = UnsubscribeRequest()
        if headers is not None:
            request.headers = headers
        request.transaction_id = transaction_id

        return await self._unary_unary(
            "/onos.e2t.e2.v1beta1.SubscriptionService/Unsubscribe",
            request,
            UnsubscribeResponse,
        )


class SubscriptionAdminServiceStub(betterproto.ServiceStub):
    async def get_channel(self, *, channel_id: str = "") -> "GetChannelResponse":

        request = GetChannelRequest()
        request.channel_id = channel_id

        return await self._unary_unary(
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/GetChannel",
            request,
            GetChannelResponse,
        )

    async def list_channels(self) -> "ListChannelsResponse":

        request = ListChannelsRequest()

        return await self._unary_unary(
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/ListChannels",
            request,
            ListChannelsResponse,
        )

    async def watch_channels(
        self, *, no_replay: bool = False
    ) -> AsyncIterator["WatchChannelsResponse"]:

        request = WatchChannelsRequest()
        request.no_replay = no_replay

        async for response in self._unary_stream(
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/WatchChannels",
            request,
            WatchChannelsResponse,
        ):
            yield response

    async def get_subscription(
        self, *, subscription_id: str = ""
    ) -> "GetSubscriptionResponse":

        request = GetSubscriptionRequest()
        request.subscription_id = subscription_id

        return await self._unary_unary(
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/GetSubscription",
            request,
            GetSubscriptionResponse,
        )

    async def list_subscriptions(self) -> "ListSubscriptionsResponse":

        request = ListSubscriptionsRequest()

        return await self._unary_unary(
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/ListSubscriptions",
            request,
            ListSubscriptionsResponse,
        )

    async def watch_subscriptions(
        self, *, no_replay: bool = False
    ) -> AsyncIterator["WatchSubscriptionsResponse"]:

        request = WatchSubscriptionsRequest()
        request.no_replay = no_replay

        async for response in self._unary_stream(
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/WatchSubscriptions",
            request,
            WatchSubscriptionsResponse,
        ):
            yield response


class ControlServiceBase(ServiceBase):
    async def control(
        self, headers: "RequestHeaders", message: "ControlMessage"
    ) -> "ControlResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_control(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "headers": request.headers,
            "message": request.message,
        }

        response = await self.control(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.e2t.e2.v1beta1.ControlService/Control": grpclib.const.Handler(
                self.__rpc_control,
                grpclib.const.Cardinality.UNARY_UNARY,
                ControlRequest,
                ControlResponse,
            ),
        }


class SubscriptionServiceBase(ServiceBase):
    async def subscribe(
        self,
        headers: "RequestHeaders",
        transaction_id: str,
        subscription: "SubscriptionSpec",
        transaction_timeout: timedelta,
    ) -> AsyncIterator["SubscribeResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def unsubscribe(
        self, headers: "RequestHeaders", transaction_id: str
    ) -> "UnsubscribeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_subscribe(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "headers": request.headers,
            "transaction_id": request.transaction_id,
            "subscription": request.subscription,
            "transaction_timeout": request.transaction_timeout,
        }

        await self._call_rpc_handler_server_stream(
            self.subscribe,
            stream,
            request_kwargs,
        )

    async def __rpc_unsubscribe(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "headers": request.headers,
            "transaction_id": request.transaction_id,
        }

        response = await self.unsubscribe(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.e2t.e2.v1beta1.SubscriptionService/Subscribe": grpclib.const.Handler(
                self.__rpc_subscribe,
                grpclib.const.Cardinality.UNARY_STREAM,
                SubscribeRequest,
                SubscribeResponse,
            ),
            "/onos.e2t.e2.v1beta1.SubscriptionService/Unsubscribe": grpclib.const.Handler(
                self.__rpc_unsubscribe,
                grpclib.const.Cardinality.UNARY_UNARY,
                UnsubscribeRequest,
                UnsubscribeResponse,
            ),
        }


class SubscriptionAdminServiceBase(ServiceBase):
    async def get_channel(self, channel_id: str) -> "GetChannelResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_channels(self) -> "ListChannelsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_channels(
        self, no_replay: bool
    ) -> AsyncIterator["WatchChannelsResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_subscription(self, subscription_id: str) -> "GetSubscriptionResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_subscriptions(self) -> "ListSubscriptionsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def watch_subscriptions(
        self, no_replay: bool
    ) -> AsyncIterator["WatchSubscriptionsResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_channel(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "channel_id": request.channel_id,
        }

        response = await self.get_channel(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_channels(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.list_channels(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch_channels(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "no_replay": request.no_replay,
        }

        await self._call_rpc_handler_server_stream(
            self.watch_channels,
            stream,
            request_kwargs,
        )

    async def __rpc_get_subscription(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "subscription_id": request.subscription_id,
        }

        response = await self.get_subscription(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_list_subscriptions(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.list_subscriptions(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_watch_subscriptions(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "no_replay": request.no_replay,
        }

        await self._call_rpc_handler_server_stream(
            self.watch_subscriptions,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/GetChannel": grpclib.const.Handler(
                self.__rpc_get_channel,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetChannelRequest,
                GetChannelResponse,
            ),
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/ListChannels": grpclib.const.Handler(
                self.__rpc_list_channels,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListChannelsRequest,
                ListChannelsResponse,
            ),
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/WatchChannels": grpclib.const.Handler(
                self.__rpc_watch_channels,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchChannelsRequest,
                WatchChannelsResponse,
            ),
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/GetSubscription": grpclib.const.Handler(
                self.__rpc_get_subscription,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetSubscriptionRequest,
                GetSubscriptionResponse,
            ),
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/ListSubscriptions": grpclib.const.Handler(
                self.__rpc_list_subscriptions,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListSubscriptionsRequest,
                ListSubscriptionsResponse,
            ),
            "/onos.e2t.e2.v1beta1.SubscriptionAdminService/WatchSubscriptions": grpclib.const.Handler(
                self.__rpc_watch_subscriptions,
                grpclib.const.Cardinality.UNARY_STREAM,
                WatchSubscriptionsRequest,
                WatchSubscriptionsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/mlb/mlb.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


@dataclass(eq=False, repr=False)
class GetMlbParamRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetMlbParamResponse(betterproto.Message):
    interval: int = betterproto.int32_field(1)
    overload_threshold: int = betterproto.int32_field(2)
    target_threshold: int = betterproto.int32_field(3)
    delta_ocn: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class SetMlbParamRequest(betterproto.Message):
    interval: int = betterproto.int32_field(1)
    overload_threshold: int = betterproto.int32_field(2)
    target_threshold: int = betterproto.int32_field(3)
    delta_ocn: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class SetMlbParamResponse(betterproto.Message):
    success: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class GetOcnRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GetOcnResponse(betterproto.Message):
    ocn_map: Dict[str, "OcnRecord"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass(eq=False, repr=False)
class OcnRecord(betterproto.Message):
    ocn_record: Dict[str, int] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_INT32
    )


@dataclass(eq=False, repr=False)
class OcnIDs(betterproto.Message):
    pass


class MlbStub(betterproto.ServiceStub):
    async def get_mlb_params(self) -> "GetMlbParamResponse":

        request = GetMlbParamRequest()

        return await self._unary_unary(
            "/onos.mlb.Mlb/GetMlbParams", request, GetMlbParamResponse
        )

    async def set_mlb_params(
        self,
        *,
        interval: int = 0,
        overload_threshold: int = 0,
        target_threshold: int = 0,
        delta_ocn: int = 0,
    ) -> "SetMlbParamResponse":

        request = SetMlbParamRequest()
        request.interval = interval
        request.overload_threshold = overload_threshold
        request.target_threshold = target_threshold
        request.delta_ocn = delta_ocn

        return await self._unary_unary(
            "/onos.mlb.Mlb/SetMlbParams", request, SetMlbParamResponse
        )

    async def get_ocn(self) -> "GetOcnResponse":

        request = GetOcnRequest()

        return await self._unary_unary("/onos.mlb.Mlb/GetOcn", request, GetOcnResponse)


class MlbBase(ServiceBase):
    async def get_mlb_params(self) -> "GetMlbParamResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_mlb_params(
        self,
        interval: int,
        overload_threshold: int,
        target_threshold: int,
        delta_ocn: int,
    ) -> "SetMlbParamResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_ocn(self) -> "GetOcnResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_mlb_params(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_mlb_params(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_set_mlb_params(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "interval": request.interval,
            "overload_threshold": request.overload_threshold,
            "target_threshold": request.target_threshold,
            "delta_ocn": request.delta_ocn,
        }

        response = await self.set_mlb_params(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_get_ocn(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.get_ocn(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.mlb.Mlb/GetMlbParams": grpclib.const.Handler(
                self.__rpc_get_mlb_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetMlbParamRequest,
                GetMlbParamResponse,
            ),
            "/onos.mlb.Mlb/SetMlbParams": grpclib.const.Handler(
                self.__rpc_set_mlb_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                SetMlbParamRequest,
                SetMlbParamResponse,
            ),
            "/onos.mlb.Mlb/GetOcn": grpclib.const.Handler(
                self.__rpc_get_ocn,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetOcnRequest,
                GetOcnResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: onos/config/diags/diags.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class Type(betterproto.Enum):
    """Change (Network or Device) event type"""

    # NONE indicates this response does not represent a modification of the
    # Change
    NONE = 0
    # ADDED is an event which occurs when a Change is added to the topology
    ADDED = 1
    # UPDATED is an event which occurs when a Change is updated
    UPDATED = 2
    # REMOVED is an event which occurs when a Change is removed from the
    # configuration
    REMOVED = 3


@dataclass(eq=False, repr=False)
class OpStateRequest(betterproto.Message):
    """
    OpStateRequest is a message for specifying GetOpState query parameters.
    """

    # The request is always in the context of a Device ID. If the device does not
    # exist or is disconnected an error will be returned.
    device_id: str = betterproto.string_field(1)
    # subscribe indicates whether to subscribe to events (e.g. ADD, UPDATE, and
    # REMOVE) that occur after all paths for the device have been streamed to the
    # client
    subscribe: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class OpStateResponse(betterproto.Message):
    # type is the type of the event
    type: "_admin__.Type" = betterproto.enum_field(1)
    # device is the device on which the event occurred
    pathvalue: "_change_device__.PathValue" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ListNetworkChangeRequest(betterproto.Message):
    """
    ListNetworkChangeRequest requests a stream of changes and updates to them
    By default, the request requests a stream of all changes that are present
    in the topology when the request is received by the service. However, if
    `subscribe` is `true`, the stream will remain open after all changes have
    been sent and events that occur following the last changes will be streamed
    to the client until the stream is closed. If "withoutReplay" is true then
    only changes that happen after the call will be returned
    """

    # subscribe indicates whether to subscribe to events (e.g. ADD, UPDATE, and
    # REMOVE) that occur after all devices have been streamed to the client
    subscribe: bool = betterproto.bool_field(1)
    # option to specify a specific network change - if blank or '*' then select
    # all Can support `*` (match many chars) or '?' (match one char) as wildcard
    changeid: str = betterproto.string_field(2)
    # option to request only changes that happen after the call
    without_replay: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class ListNetworkChangeResponse(betterproto.Message):
    """ListNetworkChangeResponse carries a single network change event"""

    # change is the network change on which the event occurred
    change: "_change_network__.NetworkChange" = betterproto.message_field(1)
    # type is a qualification of the type of change being made
    type: "Type" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class ListDeviceChangeRequest(betterproto.Message):
    """
    ListDeviceChangeRequest requests a stream of changes and updates to them By
    default, the request requests a stream of all changes that are present in
    the topology when the request is received by the service. However, if
    `subscribe` is `true`, the stream will remain open after all changes have
    been sent and events that occur following the last changes will be streamed
    to the client until the stream is closed. If "withoutReplay" is true then
    only changes that happen after the call will be returned
    """

    # subscribe indicates whether to subscribe to events (e.g. ADD, UPDATE, and
    # REMOVE) that occur after all devices have been streamed to the client
    subscribe: bool = betterproto.bool_field(1)
    # option to specify a specific device change - if blank or '*' then select
    # all Can support `*` (match many chars) or '?' (match one char) as wildcard
    device_id: str = betterproto.string_field(2)
    # device_version is an optional device version
    device_version: str = betterproto.string_field(3)
    # option to request only changes that happen after the call
    without_replay: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class ListDeviceChangeResponse(betterproto.Message):
    """ListDeviceChangeResponse carries a single network change event"""

    # change is the device change on which the event occurred
    change: "_change_device__.DeviceChange" = betterproto.message_field(1)
    # type is a qualification of the type of change being made
    type: "Type" = betterproto.enum_field(2)


class ChangeServiceStub(betterproto.ServiceStub):
    async def list_network_changes(
        self,
        *,
        subscribe: bool = False,
        changeid: str = "",
        without_replay: bool = False,
    ) -> AsyncIterator["ListNetworkChangeResponse"]:

        request = ListNetworkChangeRequest()
        request.subscribe = subscribe
        request.changeid = changeid
        request.without_replay = without_replay

        async for response in self._unary_stream(
            "/onos.config.diags.ChangeService/ListNetworkChanges",
            request,
            ListNetworkChangeResponse,
        ):
            yield response

    async def list_device_changes(
        self,
        *,
        subscribe: bool = False,
        device_id: str = "",
        device_version: str = "",
        without_replay: bool = False,
    ) -> AsyncIterator["ListDeviceChangeResponse"]:

        request = ListDeviceChangeRequest()
        request.subscribe = subscribe
        request.device_id = device_id
        request.device_version = device_version
        request.without_replay = without_replay

        async for response in self._unary_stream(
            "/onos.config.diags.ChangeService/ListDeviceChanges",
            request,
            ListDeviceChangeResponse,
        ):
            yield response


class OpStateDiagsStub(betterproto.ServiceStub):
    async def get_op_state(
        self, *, device_id: str = "", subscribe: bool = False
    ) -> AsyncIterator["OpStateResponse"]:

        request = OpStateRequest()
        request.device_id = device_id
        request.subscribe = subscribe

        async for response in self._unary_stream(
            "/onos.config.diags.OpStateDiags/GetOpState",
            request,
            OpStateResponse,
        ):
            yield response


class ChangeServiceBase(ServiceBase):
    async def list_network_changes(
        self, subscribe: bool, changeid: str, without_replay: bool
    ) -> AsyncIterator["ListNetworkChangeResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_device_changes(
        self, subscribe: bool, device_id: str, device_version: str, without_replay: bool
    ) -> AsyncIterator["ListDeviceChangeResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_list_network_changes(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "subscribe": request.subscribe,
            "changeid": request.changeid,
            "without_replay": request.without_replay,
        }

        await self._call_rpc_handler_server_stream(
            self.list_network_changes,
            stream,
            request_kwargs,
        )

    async def __rpc_list_device_changes(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "subscribe": request.subscribe,
            "device_id": request.device_id,
            "device_version": request.device_version,
            "without_replay": request.without_replay,
        }

        await self._call_rpc_handler_server_stream(
            self.list_device_changes,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.config.diags.ChangeService/ListNetworkChanges": grpclib.const.Handler(
                self.__rpc_list_network_changes,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListNetworkChangeRequest,
                ListNetworkChangeResponse,
            ),
            "/onos.config.diags.ChangeService/ListDeviceChanges": grpclib.const.Handler(
                self.__rpc_list_device_changes,
                grpclib.const.Cardinality.UNARY_STREAM,
                ListDeviceChangeRequest,
                ListDeviceChangeResponse,
            ),
        }


class OpStateDiagsBase(ServiceBase):
    async def get_op_state(
        self, device_id: str, subscribe: bool
    ) -> AsyncIterator["OpStateResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_op_state(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "device_id": request.device_id,
            "subscribe": request.subscribe,
        }

        await self._call_rpc_handler_server_stream(
            self.get_op_state,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/onos.config.diags.OpStateDiags/GetOpState": grpclib.const.Handler(
                self.__rpc_get_op_state,
                grpclib.const.Cardinality.UNARY_STREAM,
                OpStateRequest,
                OpStateResponse,
            ),
        }


from .. import admin as _admin__
from ..change import device as _change_device__
from ..change import network as _change_network__
